@@ -39,12 +39,14 @@
 ## Reference: Carl de Boor (1978), A Practical Guide to Splines, Springer, Chapter XIV
 ##
 ## @end deftypefn
-## @seealso{spline, csapi, ppval, dedup, csaps_sel}
+## @seealso{spline, csapi, ppval, dedup, bin_values, csaps_sel}
 
 ## Author: Nir Krakauer <nkrakauer@ccny.cuny.edu>
 
 function [ret,p]=csaps(x,y,p,xi,w)
 
+warning ("off", "Octave:broadcast", "local");
+
   if(nargin < 5)
     w = [];
     if(nargin < 4)
@@ -61,9 +63,13 @@ function [ret,p]=csaps(x,y,p,xi,w)
     w = w';
   endif
 
+  if any (isnan ([x y w](:)) )
+    error('NaN values in inputs; pre-process to remove them')
+  endif
+
   h = diff(x);
   if any(h <= 0)
-	error('x must be strictly increasing; use dedup to achieve this')
+	  error('x must be strictly increasing; pre-process to achieve this')
   endif
 
 
@@ -95,15 +101,9 @@ function [ret,p]=csaps(x,y,p,xi,w)
   aa = bb = cc = dd = zeros (n+1, m);
   aa(2:end, :) = a;
   cc(3:n, :) = 6*p*u; #second derivative at endpoints is 0 [natural spline]
- warn_state = warning ("query", "Octave:broadcast").state;
- warning ("off", "Octave:broadcast"); #turn off warning message for automatic broadcasting
- unwind_protect
   dd(2:n, :) = diff(cc(2:(n+1), :)) ./ h;
   %bb(2:n, :) = diff(a) ./ h - (cc(2:n, :)/2).*h - (dd(2:n, :)/6).*(h.^2);
   bb(2:n, :) = diff(a) ./ h - (h/3) .* (cc(2:n, :) + cc(3:(n+1), :)/2);
- unwind_protect_cleanup
- warning (warn_state, "Octave:broadcast");
- end_unwind_protect
 
 
 ## note: add knots to either end of spline pp-form to ensure linear extrapolation
