@@ -18,10 +18,11 @@
 ## @deftypefnx{Function File}{[@var{pp} @var{p} @var{sigma2},@var{unc_y}] =} csaps_sel(@var{x}, @var{y}, [], @var{w}=[], @var{crit}=[])
 ##
 ## Cubic spline approximation with smoothing parameter estimation @*
-## Approximates [@var{x},@var{y}], weighted by @var{w} (inverse variance; if not given, equal weighting is assumed), at @var{xi}.
+## Approximately interpolates [@var{x},@var{y}], weighted by @var{w} (inverse variance; if not given, equal weighting is assumed), at @var{xi}.
 ##
 ## The chosen cubic spline with natural boundary conditions @var{pp}(@var{x}) minimizes @var{p} Sum_i @var{w}_i*(@var{y}_i - @var{pp}(@var{x}_i))^2  +  (1-@var{p}) Int @var{pp}''(@var{x}) d@var{x}.
-## A selection criterion @var{crit} is used to find a suitable value for @var{p} (between 0 and 1); possible values for @var{crit} are `aicc' (corrected Akaike information criterion, the default); `aic' (original Akaike information criterion); `gcv' (generalized cross validation). If @var{crit} is a scalar instead of a string, then @var{p} is chosen to so that the mean square scaled residual Mean_i (@var{w}_i*(@var{y}_i - @var{pp}(@var{x}_i))^2) is approximately equal to @var{crit}.
+##
+## A selection criterion @var{crit} is used to find a suitable value for @var{p} (between 0 and 1); possible values for @var{crit} are  `vm' (Vapnik's measure [Cherkassky and Mulier 2007] from statistical learning theory); `aicc' (corrected Akaike information criterion, the default); `aic' (original Akaike information criterion); `gcv' (generalized cross validation). If @var{crit} is a scalar instead of a string, then @var{p} is chosen to so that the mean square scaled residual Mean_i (@var{w}_i*(@var{y}_i - @var{pp}(@var{x}_i))^2) is approximately equal to @var{crit}.
 ##
 ## @var{x} and @var{w} should be @var{n} by 1 in size; @var{y} should be @var{n} by @var{m}; @var{xi} should be @var{k} by 1; the values in @var{x} should be distinct and in ascending order; the values in @var{w} should be nonzero.
 ##
@@ -31,6 +32,8 @@
 ##
 ## References: 
 ##
+## Vladimir Cherkassky and Filip Mulier (2007), Learning from Data: Concepts, Theory, and Methods. Wiley, Chapter 4
+##
 ## Carl de Boor (1978), A Practical Guide to Splines, Springer, Chapter XIV
 ##
 ## Clifford M. Hurvich, Jeffrey S. Simonoff, Chih-Ling Tsai (1998), Smoothing parameter selection in nonparametric regression using an improved Akaike information criterion, J. Royal Statistical Society, 60B:271-293
@@ -42,7 +45,7 @@
 ## Grace Wahba (1983), Bayesian ``confidence intervals'' for the cross-validated smoothing spline, J Royal Statistical Society, 45B:133-150
 ##
 ## @end deftypefn
-## @seealso{csaps, spline, csapi, ppval, dedup, gcvspl}
+## @seealso{csaps, spline, csapi, ppval, dedup, bin_values, gcvspl}
 
 ## Author: Nir Krakauer <nkrakauer@ccny.cuny.edu>
 
@@ -64,6 +67,15 @@ function [ret,p,sigma2,unc_y]=csaps_sel(x,y,xi,w,crit)
     w = w';
   endif
 
+  if any (isnan ([x y w](:)) )
+    error('NaN values in inputs; pre-process to remove them')
+  endif
+
+  h = diff(x);
+  if any(h <= 0)
+	  error('x must be strictly increasing; pre-process to achieve this')
+  endif
+
   n = numel(x);
   
   if isempty(w)
@@ -84,11 +96,6 @@ function [ret,p,sigma2,unc_y]=csaps_sel(x,y,xi,w,crit)
     crit = 'aicc';
   end
 
-  h = diff(x);
-  if any(h <= 0)
-	error('x must be strictly increasing; use dedup to achieve this')
-  endif
-
   #R = spdiags([h(1:end-1) 2*(h(1:end-1) + h(2:end)) h(2:end)], [-1 0 1], n-2, n-2);
   R = spdiags([h(2:end) 2*(h(1:end-1) + h(2:end)) h(1:end-1)], [-1 0 1], n-2, n-2);
 
@@ -147,11 +154,10 @@ function Hd = influence_matrix_diag_chol(p, QT, R, y, w, n)
   d = d .^ 2;
   #5 central bands in the inverse of 6*(1-p)*QT*diag(1 ./ w)*QT' + p*R
   Binv = banded_matrix_inverse(d, U, 2);
-  Hd = diag(speye(n) - (6*(1-p))*diag(1 ./ w)*QT'*Binv*QT);	
+  Hd = full(diag(speye(n) - (6*(1-p))*diag(1 ./ w)*QT'*Binv*QT));	
 endfunction
 
 function [MSR, Ht] = penalty_terms_chol(p, QT, R, y, w, n)
-  debug_on_error(true, "local")
   #LDL factorization of 6*(1-p)*QT*diag(1 ./ w)*QT' + p*R
   U = chol(6*(1-p)*QT*diag(1 ./ w)*QT' + p*R, 'upper');
   d = 1 ./ diag(U);
@@ -162,6 +168,18 @@ function [MSR, Ht] = penalty_terms_chol(p, QT, R, y, w, n)
   MSR = mean(w .* ((6*(1-p)*diag(1 ./ w)*QT'*((6*(1-p)*QT*diag(1 ./ w)*QT' + p*R) \ (QT*y)))) .^ 2);
 endfunction
 
+function J = vm(MSR, Ht, n)
+#Vapnik-Chervonenkis penalization factor or Vapnik's measure in cherkassky07, p. 129
+  p = Ht/n;
+  if p == 0
+    J = mean(log(MSR)(:)) - log(1 - sqrt(log(n)/(2*n)));
+  elseif n == 0 || (p*(1 - log(p)) + log(n)/(2*n)) >= 1
+    J = Inf;
+  else
+    J = mean(log(MSR)(:)) - log(1 - sqrt(p*(1 - log(p)) + log(n)/(2*n)));
+  endif
+endfunction
+
 function J = aicc(MSR, Ht, n)
   J = mean(log(MSR)(:)) + 2 * (Ht + 1) / max(n - Ht - 2, 0); #hurvich98, taking the average if there are multiple data sets as in woltring86 
 endfunction
@@ -217,4 +235,34 @@ endfunction
 %!assert (1 - p, 0, 1E-6);
 %!assert (sigma2, 0, 1E-10);
 %!assert (ret - y, zeros(size(y)), 1E-4);
+%!assert (ret, (csaps_sel(x,[y 2*y],x))'(:, 1), 1E-4);
 %!assert (unc_y, zeros(size(unc_y)), 1E-5);
+
+%{
+#experiments comparing different selection criteria for recovering a function sampled with standard normal noise -- aicc was consistently better than aic, but otherwise which method does best is problem-specific
+m = 1; #number of replicates available
+ni = 200; #number of evaluation points
+ns = [5 10 20 40]; #number of given sample points
+nk = 100; #number of trials to average over
+f = @(x) sin(2*pi*x); #function generating the synthetic data
+mse = nan(4, numel(ns), nk);
+for i = 1:numel(ns)
+  for k = 1:nk
+    n = ns(i);
+    x = linspace(0, 1, n)(:);
+    y = f(x) + randn(n, m);
+    xi = rand(ni, 1);
+    yt = f(xi);
+    yi = csaps_sel(x,y,xi,[],'vm');
+    mse(1, i, k) = meansq((yi - yt')(:));
+    yi = csaps_sel(x,y,xi,[],'aicc');
+    mse(2, i, k) = meansq((yi - yt')(:));
+    yi = csaps_sel(x,y,xi,[],'aic');
+    mse(3, i, k) = meansq((yi - yt')(:)); 
+    yi = csaps_sel(x,y,xi,[],'gcv');
+    mse(4, i, k) = meansq((yi - yt')(:));   
+  endfor
+endfor
+msem = mean(mse, 3);
+%}
+
